//+------------------------------------------------------------------+
//|                                                      NN TEST.mq5 |
//|                                     Copyright 2023, Omega Joctan |
//|                        https://www.mql5.com/en/users/omegajoctan |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Omega Joctan"
#property link      "https://www.mql5.com/en/users/omegajoctan"
#property version   "1.00"
#property strict

#resource "\\Files\\regressornets\\activation.bin" as double nn_activation[]
#resource "\\Files\\regressornets\\architecture.bin" as double nn_architecture[]
#resource "\\Files\\regressornets\\bias.bin" as double nn_bias[]
#resource "\\Files\\regressornets\\inputs.bin" as double nn_inputs[]
#resource "\\Files\\regressornets\\weights.bin" as double nn_weights[]

//--- Scaler

#resource "\\Files\\ohlc-reg\\mean.bin" as double scaler_mean[]
#resource "\\Files\\ohlc-reg\\std.bin" as double scaler_std[]


#include <regressornets.mqh>
CRegressorNets *nn;

#include <MatrixExtend.mqh>
#include <preprocessing.mqh>
#include <metrics.mqh>

StandardizationScaler *scaler;
vector open_, high_, low_;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   
   nn = new CRegressorNets(nn_architecture, nn_activation, nn_inputs, nn_weights, nn_bias);
   
   scaler = new StandardizationScaler(scaler_mean, scaler_std);
   
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   if (CheckPointer(nn)!=POINTER_INVALID)
     delete nn;
   if (CheckPointer(scaler)!=POINTER_INVALID)
     delete scaler;
     
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---

   open_.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_OPEN, 0, 1);
   high_.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_HIGH, 0, 1);
   low_.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_LOW, 0, 1);
   
   vector inputs= {open_[0], high_[0], low_[0]};
   
   double predicted_close = nn.predict(inputs);
   
   Comment("Predicted close: ",NormalizeDouble(predicted_close, Digits()));
  }
//+------------------------------------------------------------------+
