//+------------------------------------------------------------------+
//|                                                  TS AI- test.mq5 |
//|                                     Copyright 2023, Omega Joctan |
//|                        https://www.mql5.com/en/users/omegajoctan |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Omega Joctan"
#property link      "https://www.mql5.com/en/users/omegajoctan"
#property version   "3.00"

#resource "\\Files\\rnn.onnx" as uchar rnn[]
#resource "\\Files\\Standardization-ohlc\\mean.bin" as double scaler_mean[]
#resource "\\Files\\Standardization-ohlc\\std.bin" as double scaler_std[]

#include <MatrixExtend.mqh>
#include <preprocessing.mqh>
#include <Timeseries Deep Learning\onnx.mqh>
#include <Timeseries Deep Learning\tsdataprocessor.mqh>

StandardizationScaler *scaler;
CONNX onnx;
CTSDataProcessor ts_data_processor;
CTensors *dataset_tensor;

input int nn_time_step = 10;
vector open, high, low;
matrix dataset(nn_time_step, 3);

vector classes_ = {-1, 0, 1};
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   
   if (!onnx.Init(rnn))
     return (INIT_FAILED);
   
   scaler = new StandardizationScaler(scaler_mean, scaler_std);
   
   
   return INIT_SUCCEEDED;
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
   delete (scaler);
   delete (dataset_tensor);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   
   open.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_OPEN, 1, nn_time_step);
   high.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_HIGH,1, nn_time_step);
   low.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_LOW, 1, nn_time_step);
   
   dataset.Col(open, 0);
   dataset.Col(high, 1);
   dataset.Col(low, 2);
   
   dataset = scaler.transform(dataset); //normalized data
   
   dataset_tensor = ts_data_processor.extract_timeseries_data(dataset, nn_time_step);
   
   matrix inputs = dataset_tensor.Get(0);
   
   int next_bar_prediction = onnx.predict_bin(inputs, classes_);
   
   Comment("signal: ",next_bar_prediction);
   
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
