## Neural Networks | Read The Docs 

[![feed forward nn video](https://github.com/MegaJoctan/MQL5-Neural-Networks-solution/assets/65341461/c943cc1e-db93-45f0-946d-5f4113014df3)](https://youtu.be/y-TbXmfA8NY)

**Classes:**

* **CRegressorNets:** This class is designed for training regression models using various activation functions and loss functions.
* **CPatternNets:** This class is designed for training classification models using various activation functions and loss functions.

**Enumerations:**

* **activation:** This enumeration defines supported activation functions for neural networks (e.g., AF_RELU, AF_SIGMOID).
* **loss:** This enumeration defines supported loss functions for training models (e.g., LOSS_MSE, LOSS_BCE).

**CRegressorNets Class:**

**Public Constructors:**

* **CRegressorNets(vector &HL_NODES, activation AF_, loss LOSS_)** - Creates a new CRegressorNets object for training a regression model.
    * **HL_NODES (vector<int>)**:  Defines the number of neurons in each hidden layer.
    * **AF_ (activation)**:  Specifies the activation function for the model.
    * **LOSS_ (loss)**:  Defines the loss function used for training.

* **CRegressorNets(const double &architecture[], const double &activation_[], const double &inputs[], const double &weights[], const double &bias[])** - Loads a pre-trained regression model from provided weight and bias arrays.

**Public Destructor:**

* **~CRegressorNets(void)** - Destructor for the CRegressorNets class.

**Public Member Functions:**

* **double predict(const vector &x)** - Predicts the output for a single data point using the trained model.
* **vector<double> predict(const matrix &x)** - Predicts the output for multiple data points using the trained model.
* **void fit(const matrix &x, const vector &y, const uint epochs, Optimizer* optimizer, uint batch_size=0, bool show_batch_progress=false, regression_metrics METRIC_=METRIC_R_SQUARED, weight_intializers WEIGHT_INIT=WEIGHT_INTIALIZER_HE)** - Trains the regression model using the specified optimizer, data, and configuration options.
    * **x (matrix)**:  Input data for training.
    * **y (vector<double>)**:  Target values for training.
    * **epochs (uint)**:  Number of training epochs.
    * **optimizer (Optimizer*)**:  Pointer to an optimizer object (e.g., OptimizerAdam).
    * **batch_size (uint)**:  Optional batch size for mini-batch training (default: 0 for full batch).
    * **show_batch_progress (bool)**:  Optional flag to show training progress per batch (default: false).
    * **METRIC_ (regression_metrics)**:  Optional regression evaluation metric (default: METRIC_R_SQUARED).
    * **WEIGHT_INIT (weight_intializers)**:  Optional weight initialization method (default: WEIGHT_INTIALIZER_HE).

* **bool save_model(string dir)** - Saves the trained model weights and biases to a specified directory.

**CPatternNets Class:**

**Public Constructors:**

* **CPatternNets(vector &HL_NODES, activation AF_, loss LOSS_)** - Creates a new CPatternNets object for training a classification model.
    * **HL_NODES (vector<int>)**:  Defines the number of neurons in each hidden layer.
    * **AF_ (activation)**:  Specifies the activation function for the model.
    * **LOSS_ (loss)**:  Defines the loss function used for training.

* **CPatternNets(const double &architecture[], const double &activation_[], const double &classes[], const double &inputs[], const double &weights[], const double &bias[])** - Loads a pre-trained classification model from provided weight and bias arrays.

**Public Destructor:**

* **~CPatternNets(void)** - Destructor for the CPatternNets class.

**Public Member Functions:**

* **vector<double> predict_proba(const vector &x)** - Predicts the class probabilities for a single data point.
* **matrix<double> predict_proba(const matrix &x)** - Predicts the class probabilities for multiple data points.
* **int predict_bin(const vector &x)** - Predicts the most likely class for a single data point.
* **vector<int> predict_bin(const matrix &x)** - Predicts the most likely class for multiple data points.
* **void fit(const matrix &x, const vector &y, const uint epochs, Optimizer* optimizer


## Neural Network Optimizers

This library provides various optimizer implementations commonly used in machine learning for updating model parameters based on calculated gradients. Each optimizer class offers a `virtual void update(matrix &parameters, matrix &gradients)` function that takes the current model parameters and gradients as input and updates the parameters.

**Optimizers Included:**

- **OptimizerAdam:** Implements the Adam optimizer, an adaptive learning rate method that combines the benefits of RMSprop and AdaGrad.
- **OptimizerSGD:** Implements Stochastic Gradient Descent (SGD), a fundamental optimization algorithm that updates parameters based on individual data points.
- **OptimizerBGD:** Inherits from `OptimizerSGD` and represents Batch Gradient Descent (BGD), which updates parameters based on the entire dataset in each iteration.
- **OptimizerRMSprop:** Implements the RMSprop optimizer, which adapts the learning rate for each parameter based on its historical squared gradients.
- **OptimizerAdaGrad:** Implements the AdaGrad optimizer, which adaptively reduces the learning rate for parameters with frequent updates.
- **OptimizerAdaDelta:** Implements the AdaDelta optimizer, a variant of AdaGrad that addresses its tendency to accumulate learning rates over time.
- **OptimizerNadam:** Implements the Nadam optimizer, an extension of Adam that incorporates momentum for faster convergence.

**Common Configuration Parameters:**

Several optimizers share common configuration parameters that can be set in the constructor:

- **learning_rate (double):** Controls the step size taken during parameter updates. Defaults to 0.01.
- **beta1, beta2 (double):** Parameters for momentum calculations in optimizers like Adam and Nadam (defaults vary for each optimizer).
- **epsilon (double):** A small value added to the denominator to prevent division by zero (defaults to 1e-8).
- **decay_rate (double):** Decay rate for exponential moving averages used in some optimizers (default varies).

**Detailed Optimizer Descriptions:**

**1. OptimizerAdam:**

Theory: Adam is an adaptive learning rate optimization algorithm that estimates first and second moments of the gradients. It incorporates momentum for faster convergence and adapts learning rates based on historical squared gradients. Adam addresses some limitations of SGD and RMSprop, often leading to faster convergence and better performance.

**2. OptimizerSGD:**

Theory: SGD is a basic optimization algorithm that updates model parameters based on the gradient of the loss function with respect to a single data point. It is simple to implement, but can be slow to converge and sensitive to learning rate selection.

**3. OptimizerBGD:**

Theory: BGD is a variant of SGD that updates model parameters based on the gradient calculated using the entire dataset in each iteration. It can be computationally expensive for large datasets, and SGD is often preferred due to its more efficient updates.

**4. OptimizerRMSprop:**

Theory: RMSprop is an adaptive learning rate optimizer that addresses the issue of diminishing learning rates in SGD. It maintains an exponentially decaying average of squared gradients for each parameter and uses this average to normalize the gradient before updating the parameters. This allows for larger learning rates while maintaining stability.

**5. OptimizerAdaGrad:**

Theory: AdaGrad is an adaptive learning rate optimizer that accumulates the squared gradients over time to adapt the learning rate for each parameter. However, AdaGrad can suffer from diminishing learning rates as the accumulated gradients continue to grow.

**6. OptimizerAdaDelta:**

Theory: AdaDelta is a variant of AdaGrad that addresses its diminishing learning rate issue. It incorporates an exponential moving average of squared gradients and squared parameter updates to dynamically adjust learning rates without excessive accumulation.

**7. OptimizerNadam:**

Theory: Nadam is an extension of Adam that incorporates momentum for faster convergence. It combines the adaptive learning rate adjustments of Adam with momentum updates similar to those used in SGD with momentum. Nadam can sometimes achieve faster convergence compared to Adam.

**Usage:**

```mql5
// Example usage of OptimizerAdam
OptimizerAdam optimizer(0.001, 0.9, 0.999, 1e-8);

matrix parameters, gradients;
// ... (initialize parameters and calculate gradients)

optimizer.update(parameters, gradients);

// Use the updated parameters in your model
```

**Note:** This documentation provides a brief overview of each optimizer. Refer to relevant research papers or online resources for a more in-depth understanding of their theoretical foundations and implementation details.


**Theory for Each Optimizer:**

- Refer to online resources for detailed explanations of each optimizer algorithm:
    - Adam: [https://arxiv.org/abs/1412.6980](https://arxiv.org/abs/1412.6980)
    - SGD, BGD: [https://en.wikipedia.org/wiki/Stochastic_gradient_descent](https://en.wikipedia.org/wiki/Stochastic_gradient_descent)
    - RMSprop: [https://github.com/zoq/Awesome-Optimizer](https://github.com/zoq/Awesome-Optimizer)
    - AdaGrad, AdaDelta, Nadam: [https://arxiv.org/abs/1909.09712](https://arxiv.org/abs/1909.09712)

