## MQL5 Time Series Data Processor Documentation

[![deep learning video](https://github.com/MegaJoctan/MQL5-Neural-Networks-solution/assets/65341461/facc1d0f-b305-43a9-806f-162467afa9fe)](https://youtu.be/yQ0HKmXn824)

**Class: CTSDataProcessor**

**Public Member Functions:**

* **CTSDataProcessor(void)** - Constructor for the `CTSDataProcessor` class. Initializes member variables.
* **~CTSDataProcessor(void)** - Destructor for the `CTSDataProcessor` class. Releases any allocated memory for created `CTensors` objects.

* **CTensors* extract_timeseries_data(const matrix<double> &x, const int time_step)** - Extracts time series data from a single input matrix.
    * **x (const matrix<double>&)**:  Input data matrix.
    * **time_step (int)**:  Number of data points to include in each time series sample.
* **CTensors* extract_timeseries_data(const matrix<double> &MATRIX, vector &y, const int time_step)** - Extracts time series data from an input matrix with separate target labels (y).
    * **MATRIX (const matrix<double>&)**:  Input data matrix.
    * **y (vector<double>&)**:  Output vector containing target labels (one for each row in the matrix).
    * **time_step (int)**:  Number of data points to include in each time series sample.

**Private Members:**

* **tensor_memory (CTensors*[])**:  Internal array to store pointers to dynamically created `CTensors` objects.
* **xandysplit (bool)**:  Flag indicating whether the input matrix contains separate x and y data (true) or just x data (false).

**Functionality:**

* The `CTSDataProcessor` class helps prepare time series data for use with machine learning models.
* The `extract_timeseries_data` functions take an input data matrix and a time step parameter.
* The time step defines the number of consecutive data points to include in each sample of the resulting time series.
* The first function assumes the input matrix only contains independent variables (x).
* The second function assumes the input matrix contains separate columns for independent (x) and dependent variables (y). It also takes a vector to store the extracted target labels.
* Both functions iterate through the input data, creating new time series samples by concatenating rows within the specified time step window.

**Destructor:**

* The destructor ensures proper memory management by checking and deleting any dynamically allocated `CTensors` objects stored in the `tensor_memory` array.


## MQL5 ONNX Model Inference Library Documentation

This document describes the `CONNX` class, which provides functionalities for loading and using pre-trained ONNX models for making predictions within MQL5 environments.

**Class: CONNX**

**Public Member Functions:**

* **CONNX(void)** - Constructor for the `CONNX` class. Initializes member variables.
* **~CONNX(void)** - Destructor for the `CONNX` class. Releases any allocated resources.

* **bool Init(const uchar &onnx_buff[], ulong flags=ONNX_DEFAULT)** - Initializes the ONNX model from a provided byte buffer containing the model data.
    * **onnx_buff (const uchar[])**:  Pointer to the byte buffer containing the ONNX model data.
    * **flags (ulong)**:  Optional flags for model loading (default: ONNX_DEFAULT).

* **virtual int predict_bin(const matrix &x, const vector &classes_in_data)** - Predicts the most likely class label for a single data point in a classification problem.
    * **x (matrix)**:  Input data for prediction.
    * **classes_in_data (vector<string>)**:  Optional vector containing class labels (used for interpretation).
* **virtual vector<double> predict_bin(CTensors &timeseries_tensor, const vector &classes_in_data)** - Predicts the most likely class labels for multiple data points in a time series classification problem. 
    * **timeseries_tensor (CTensors&)**:  Time series data tensor for prediction.
    * **classes_in_data (vector<string>)**:  Optional vector containing class labels (used for interpretation).

* **double predict(const matrix &x)** - Predicts the output value for a single data point in a regression problem.
    * **x (matrix)**:  Input data for prediction.

**Private Members:**

* **initialized (bool)**:  Flag indicating whether the model has been successfully initialized.
* **onnx_handle (long)**:  Internal handle for the loaded ONNX model.
* **inputs[], outputs[] (long[])**:  Arrays storing the handle IDs for model inputs and outputs.

**Functionality:**

* The `CONNX` class allows loading pre-trained ONNX models into the MQL5 environment.
* The `Init` function takes a byte buffer containing the model data and optionally flags for model loading.
* The `predict_bin` function (with matrix input) is used for classification tasks on single data points. It returns the predicted class label (integer index). An optional vector of class labels can be provided for interpretation.
* The `predict_bin` function (with CTensors input) is used for classification tasks on time series data. It returns a vector of predicted class labels for each data point in the time series. An optional vector of class labels can be provided for interpretation.
* The `predict` function is used for regression tasks on single data points. It returns the predicted continuous output value.
