# MQL5 Neural Networks Solution

## Desktop Application Configuration Reference

![bandicam 2024-03-12 16-27-22-306](https://github.com/MegaJoctan/MQL5-Neural-Networks-solution/assets/65341461/6809371f-6826-4b16-8250-52957b7db05f)

This desktop app is located under [sandman-release-2](https://github.com/MegaJoctan/MQL5-Neural-Networks-solution/releases/tag/v2.0.0) under The Assets section or you can [Download it](https://github.com/MegaJoctan/MQL5-Neural-Networks-solution/releases/download/v2.0.0/pyMT5_v2.0.0_win_setup.exe) directly

**Data Configuration:**

* **TARGET_COLUMN:** (String) This specifies the name of the column in your data that contains the target variable you want to predict (e.g., closing stock price, next day's temperature).

**Model Architecture:**

* **TIME_STEP:** (Integer) This defines the length of the sequence that the model will process as a single input. For example, a `TIME_STEP` of 10 means the model will consider the past 10 data points to predict the next value.
* **NEURONS:** (Integer) This specifies the number of neurons in each hidden layer of the RNN or LSTM model. Higher values can increase model complexity but might also lead to overfitting.
* **LAYERS:** (Integer) This defines the number of hidden layers stacked in your RNN or LSTM architecture. More layers can potentially capture more complex relationships in the data, but again, can increase overfitting risk.

**Training Configuration:**

* **DROPOUT_RATE:** (Float between 0 and 1) This is a regularization technique that randomly drops out a certain percentage of neurons during training. It helps prevent overfitting by reducing the reliance on any specific neuron and encouraging the model to learn more robust features.
* **BATCH_SIZE:** (Integer) This determines the number of data samples processed together during a single training step. Larger batch sizes can improve training speed but might lead to less precise updates, while smaller batch sizes can be slower but potentially more accurate.
* **EPOCHS:** (Integer) This represents the number of times the entire training dataset is passed through the model during training. Increasing epochs allows the model to learn more complex relationships, but too many epochs can lead to overfitting.
* **ALPHA (Learning Rate):** (Float) This controls the step size taken by the optimizer during gradient descent. A higher learning rate can lead to faster learning but might cause the model to jump over optimal solutions. A lower learning rate ensures more gradual convergence but might take longer to train.
* **PATIENCE:** (Integer) This is used in early stopping, a technique that monitors validation performance during training. If the validation loss doesn't improve for `PATIENCE` consecutive epochs, training stops to prevent overfitting.
* **RANDOM_STATE:** (Integer) This sets a seed for the random number generator used during training. Fixing the seed ensures reproducibility, meaning you'll get the same results when you run the training process again with the same data and configuration.

**Overall Impact:**

These configurations work together to define the structure and training behavior of your RNN or LSTM model. Adjusting these values can significantly influence the model's performance and ability to capture temporal dependencies in your time series data.


**MatrixExtend(MatrixExtend.mqh)**

The `MatrixExtend` class in MQL5 extends the capabilities of the built-in MetaTrader 5 `matrix` class, providing a diverse array of utility functions and methods for efficient matrix manipulation, data transformations, and machine learning tasks within your trading strategies. This class aims to streamline your MQL5 coding experience by encapsulating common matrix operations and offering convenient functionalities.

**Key Functionalities**

* **File I/O:**
    * `WriteCsv(csv_name, matrix_, header[], bool common=false, int digits=5)`: Writes a matrix to a CSV file with optional headers and formatting control.
    * `WriteCsv(csv_name, matrix_, string header_string="", bool common=false, int digits=5)`: Alternative version with a single header string.
    * `ReadCsv(string file_name, string &headers, string delimiter=",", bool common=false)`: Reads a CSV file into a matrix, extracting headers.
    * `DBtoMatrix(int db_handle, string table_name, string &column_names[], int total=WHOLE_ARRAY)`: Retrieves data from a MetaTrader 5 database table into a matrix.
* **Manipulations:**
    * `RemoveCol(matrix &mat, ulong col)`: Removes a specified column from a matrix.
    * `RemoveMultCols(matrix &mat, int &cols[])`: Removes multiple columns from a matrix using an integer array of column indices.
    * `RemoveMultCols(matrix &mat, int from, int total=WHOLE_ARRAY)`: Removes a range of consecutive columns from a matrix.
    * `RemoveRow(matrix &mat, ulong row)`: Removes a specified row from a matrix.
    * `VectorRemoveIndex(vector &v, ulong index)`: Removes an element from a vector at a given index.
* **Machine Learning:**
    * `XandYSplitMatrices(const matrix<T> &matrix_, matrix<T> &xmatrix, vector<T> &y_vector, int y_column=-1)`: Splits a matrix into separate feature (X) and target (y) matrices for machine learning.
    * `TrainTestSplitMatrices(const matrix<T> &matrix_, matrix<T> &x_train, vector<T> &y_train, matrix<T> &x_test, vector<T> &y_test, double train_size=0.7, int random_state=-1)`: Splits a matrix into training and testing sets for machine learning, with optional randomization.
    * `DesignMatrix(matrix &x_matrix)`: Creates a design matrix (one-hot encoded) from a categorical feature matrix.
    * `OneHotEncoding(const vector &v)`: Performs one-hot encoding on a vector of categorical values.
    * `Sign(matrix &x)` and `Sign(vector &x)`: Applies the sign function (element-wise) to a matrix or vector.
    * `eye(uint num_features)`: Generates an identity matrix of the specified size.
* **Detection:**
    * `Unique(const string &Array[], string &classes_arr[])` and `Unique(const vector &v)`: Identifies unique elements (classes) within an array or vector.
    * `Unique_count(vector &v)`: Counts the occurrences of each unique element in a vector.
* **Random Data Generation:**
    * `Random(T min, T max, int size, int random_state=-1)`: Generates a random vector of a specified type with optional random seed.
    * `Random(double min, double max, ulong rows, ulong cols, int random_state=-1)`: Generates a random matrix within a specified range.
* **Search:**
    * `Search(const vector<T> &v, T value)`: Searches for a value within a vector and returns its indices.
* **Transformations:**
    * `VectorToMatrix(const vector &v, ulong cols=1)`: Converts a vector into a matrix with a specified number of columns.
    * `MatrixToVector(const matrix<T> &mat)`: Converts a matrix into a vector.
    * `ArrayToVector(const T &Arr[])` and `VectorToArray(const vector<T> &v, T &arr[])`: Converts between arrays and vectors.
* **Manipulations:**
    * `concatenate(const matrix &mat1, const matrix &mat2, int axis = 0)`: Concatenates two matrices along a specified axis (0 for rows, 1 for columns).
    * `concatenate(const matrix<T> &mat, const vector<T> &v, int axis=1)`: Concatenates a matrix and a vector along a specified axis.
    * `Copy(const vector<T> &src, vector<T> &dst, ulong src_start, ulong total=WHOLE_ARRAY)`: Copies a portion of a source vector to a destination vector.
    * `Reverse(vector<T> &v)` and `Reverse(matrix<T> &mat)`: Reverses the order of elements in a vector or matrix.
    * `HadamardProduct(matrix &a, matrix &b)`: Performs element-wise multiplication between two matrices (Hadamard product).

* **Randomization:**
    * `Randomize(vector<T> &v, int random_state=-1, bool replace=false)` and `Randomize(matrix<T> &matrix_, int random_state=-1, bool replace=false)`: Randomizes elements in a vector or matrix with optional replacement.

* **Normalization:**
    * `NormalizeDouble_(vector<T> &v, int digits=3)` and `NormalizeDouble_(matrix<T> &mat, int digits=3)`: Normalizes double-precision values in a vector or matrix to a specified number of decimal places.

* **Data Access:**
    * `CopyBufferVector(int handle, int buff_num, int start_pos, int count, vector &v)`: Copies data from a MetaTrader 5 buffer to a vector.
    * `Stringfy(vector &v, int digits = 2)`: Converts a vector into a string representation.
    * `Zeros(ulong rows, ulong cols)` and `Zeros(ulong size)`: Creates zero matrices or vectors of specified dimensions.
    * `Get(const matrix &mat, ulong start_index, ulong end_index)` and `Get(const vector &v, ulong start_index, ulong end_index)`: Extracts a sub-matrix or sub-vector based on starting and ending indices.

* **Sorting:**
    * `Sort(vector<T> &v, ENUM_SORT_MODE sort_mode=SORT_ASCENDING)`: Sorts a vector in ascending or descending order.
    * `ArgSort(vector<T> &v)`: Returns the indices of the sorted elements in a vector.

* **Others:**
    * `PrintShort(matrix &matrix_, ulong rows=5, int digits=5)`: Prints a concise representation of a matrix, showing the first `rows` rows with `digits` decimal places.

**Important Notes:**

* The `MatrixExtend` class provides template-based functions, allowing you to use it with various data types (e.g., `double`, `int`, `string`).
* For detailed information and usage examples of each function, refer to the MQL5 documentation or the code itself.



## Preprocessing.mqh: MQL5 Normalization Techniques

This MQL5 library, `preprocessing.mqh`, provides three essential normalization techniques for machine learning:

* **RobustScaler**
* **MinMaxScaler**
* **StandardizationScaler**
* **CLabelEncoder** (for label encoding)

These functions help prepare your data for machine learning algorithms by scaling or encoding features to a common range or representation. This step is crucial for improving the performance and stability of many machine learning models.

**Key Functionalities:**

**Scalers:**

* **RobustScaler:**
    * Uses the median and interquartile range (IQR) for scaling.
    * Less prone to outliers compared to other scalers.
* **MinMaxScaler:**
    * Scales features to a range between `0` and `1`.
    * Useful for algorithms sensitive to feature scales.
* **StandardizationScaler:**
    * Standardizes features by subtracting the mean and dividing by the standard deviation.
    * Leads to features with zero mean and unit variance.

**Label Encoder:**

* **CLabelEncoder:**
    * Encodes categorical string labels into numerical labels.
    * Useful for converting categorical features into numerical representations usable by machine learning models.

**Usage:**

Each scaler class provides:

* `fit_transform(const matrix &X)`: Fits the scaler on the data matrix `X` and performs the transformation.
* `transform(const matrix &X)`: Transforms the data matrix `X` using the fitted scaler.
* `transform(const vector &X)`: Transforms the data vector `X` using the fitted scaler.

The `CLabelEncoder` class offers:

* `encode(string &Arr[])`: Encodes a string array into a numerical vector.

**Saving and Loading Models (Scalers):**

* Each scaler class supports saving and loading models:
    * `save(string save_dir, string column_names, bool common_dir=false)`: Saves the scaler model to a csv file.
    * `load(string dir)`: Loads a previously saved scaler model from a csv file.

**Documentation and Examples:**

For detailed usage examples and full documentation, please refer to the comments within the `preprocessing.mqh` library file.

**Additional Notes:**

* Remember to include this library file (`#include "preprocessing.mqh"`) in your MQL5 code where you want to use its functionalities.



## Metrics.mqh: MQL5 Library for Evaluation Metrics

The `metrics.mqh` library provides an array of useful functions for calculating and evaluating various performance metrics in MQL5, particularly for machine learning tasks. These metrics help assess the effectiveness of your model's predictions compared to the ground truth (actual values).

**Metrics Supported:**

* **Regression Metrics:**
    * R-squared
    * Adjusted R-squared
    * Residual Sum of Squares (RSS)
    * Mean Squared Error (MSE)
    * Root Mean Squared Error (RMSE)
    * Mean Absolute Error (MAE)
* **Classification Metrics:**
    * Accuracy score
    * Accuracy per class
    * Precision per class
    * Recall per class
    * F1-score per class
    * Specificity per class
    * Receiver Operating Characteristic (ROC) curve

**Functionalities:**

1. **Regression Metrics:**
    * `r_squared(const vector &True, const vector &Pred)`: Calculates the R-squared value for regression models.
    * `adjusted_r(const vector &True, const vector &Pred, uint indep_vars = 1)`: Calculates the adjusted R-squared value.
    * `rss(const vector &True, const vector &Pred)`: Calculates the Residual Sum of Squares.
    * `mse(const vector &True, const vector &Pred)`: Calculates the Mean Squared Error.
    * `rmse(const vector &True, const vector &Pred)`: Calculates the Root Mean Squared Error.
    * `mae(const vector &True, const vector &Pred)`: Calculates the Mean Absolute Error.
    * `RegressionMetric(const vector &True, const vector &Pred, regression_metrics METRIC_)`: Calculates a specific regression metric based on the provided enum value (`METRIC_`).

2. **Classification Metrics:**
    * `accuracy_score(const vector &True, const vector &Pred)`: Calculates the overall accuracy (correctly classified instances).
    * `accuracy(const vector &True, const vector &Preds)`: Calculates the accuracy per class (vector of values).
    * `precision(const vector &True, const vector &Preds)`: Calculates the precision per class (vector of values).
    * `recall(const vector &True, const vector &Preds)`: Calculates the recall per class (vector of values).
    * `f1_score(const vector &True, const vector &Preds)`: Calculates the F1-score per class (vector of values).
    * `specificity(const vector &True, const vector &Preds)`: Calculates the specificity per class (vector of values).
    * `roc_curve(const vector &True, const vector &Preds, bool show_roc_curve=false)`: Calculates and returns an `roc_curve_struct` containing True Positive Rates (TPR), False Positive Rates (FPR), and thresholds. Optional argument to display the ROC curve graphically.
    * `classification_report(const vector &True, const vector &Pred, bool show_roc_curve=false)`: Generates a comprehensive classification report with various metrics and optionally displays the ROC curve.

3. **Helper Functions:**
    * `SearchPatterns(const vector &True, int value_A, const vector &B, int value_B)`: Internal helper function used for calculations.

**Additional Notes:**

* The `vector` type is commonly used to represent data in MQL5.
* Ensure correct data sizes and types when using these functions (e.g., `True` and `Pred` vectors should have the same length).
* Refer to the `metrics.mqh` file for the specific implementation and detailed comments within the code for more nuanced understanding.

**Usage Example (Regression):**

```MQL5
// Example: Calculating R-squared for a regression model

vector true_values;
vector predicted_values;

// ... populate vectors with true and predicted values ...

double r2_score = Metrics::r_squared(true_values, predicted_values);

Print("R-squared:", r2_score);
```

**Usage Example (Classification):**

```MQL5
// Example: Calculating accuracy and F1-score per class

vector true_labels;
vector predicted_labels;

// ... populate vectors with true and predicted labels ...

vector accuracy_per_class = Metrics::accuracy(true_labels, predicted_labels);
vector f1_score_per_class = Metrics::f1_score(true_labels, predicted_labels);

Print("Accuracy per class:", accuracy_per_class);
Print("F1-score per class:",
```


## Kohonen Maps.mqh: MQL5 Library for Self-Organizing Feature Maps (SOMs)

The `kohonen_maps.mqh` library implements Kohonen Maps, also known as Self-Organizing Maps (SOMs), for MQL5. This unsupervised learning technique helps visualize and analyze high-dimensional data by projecting it onto a lower-dimensional space while preserving the topological structure.

**Key Functionalities:**

* **Constructor:**
    * `CKohonenMaps(matrix &matrix_, uint random_state=42, uint clusters=2, double alpha=0.01, uint epochs=100)`: Initializes the Kohonen Maps object with:
        * `matrix_`: The input data matrix.
        * `random_state`: Optional random seed (default: 42).
        * `clusters`: The number of desired clusters (default: 2).
        * `alpha`: The learning rate (default: 0.01).
        * `epochs`: The number of training epochs (default: 100).
* **Prediction:**
    * `uint predict(vector &v)`: Predicts the cluster label for a given data vector `v`.
    * `vector predict(matrix &matrix_)`: Predicts cluster labels for all rows in the input matrix `matrix_`, returning a vector of labels.

**Internals (Private Members):**

* `n`: Number of features (columns) in the input data.
* `m`: Number of clusters (output neurons).
* `rows`: Number of rows in the input data.
* `Matrix`: The input data matrix (copied internally).
* `c_matrix`: The cluster matrix (stores cluster centers).
* `w_matrix`: The weights matrix (weights for each cluster and feature).
* `w_vector`: Internal representation of weight vectors (optional, not exposed publicly).
* `o_matrix`: The output layer matrix (used during training, not exposed publicly).

**Helper Functions (Protected Members):**

* `Euclidean_distance(const vector &v1, const vector &v2)`: Calculates the Euclidean distance between two vectors.
* `CalcTimeElapsed(double seconds)`: Calculates and returns a string representing the elapsed time in a human-readable format.

**Note:** This documentation provides an overview of the public functionalities and internal data structures. The specific training and prediction algorithms are not explicitly detailed here, as they involve complex calculations and iterative updates. Refer to the `kohonen_maps.mqh` file for the full implementation and code comments for in-depth understanding.




## Tensors.mqh: MQL5 Library for Multidimensional Data Representation

The `Tensors.mqh` library introduces the `CTensors` class, offering a structured approach to handling multidimensional data in MQL5. This class can be particularly beneficial for machine learning applications that often involve working with data in higher dimensions.

**Key Functionalities:**

* **Constructor:**
    * `CTensors(uint DIM)`: Creates a new `CTensors` object with the specified number of dimensions (`DIM`).

**Data Storage:**

* The `CTensors` class internally utilizes an array of `CMatrix` objects (`matrices[]`). Each `CMatrix` instance holds a single MQL5 `matrix` object, effectively creating a multidimensional structure.

**Public Methods:**

* **Add (template):**
    * Adds an `matrix<T>` (matrix of type `T`) to the `CTensors` object at a specified position (`POS`).
    * Supports different data types (`T`) through templating.
* **Append (template):**
    * Appends a vector of type `T` to a specific dimension (`POS`) of the `CTensors` object.
    * Useful for creating tensors from data stored in individual vectors.
* **Print_():**
    * Prints a human-readable representation of the contents of the `CTensors` object.
* **Get(ulong POS):**
    * Retrieves the `matrix` object at a specified position (`POS`) within the `CTensors` structure.
* **Fill (template):**
    * Fills all elements of the `CTensors` object with a specified value (`value`) of type `T`.
* **MemoryClear():**
    * Releases memory allocated for the internal `CMatrix` objects to avoid memory leaks.

**Note:**

* The provided documentation serves as an overview. The specific implementation details and error handling mechanisms within the `CTensors` class are not explicitly explained here.
* Refer to the `Tensors.mqh` file for the full implementation and code comments for a comprehensive understanding.

**Usage Example:**

```mql5
// Example: Creating a 2D tensor and adding data

CTensors tensor(2); // 2 dimensions

matrix<double> mat_1 = {{1.2, 3.4}, {5.6, 7.8}};
vector<int> vec_2 = {10, 20, 30};

tensor.Add(mat_1, 0); // Add matrix to the first dimension
tensor.Append(vec_2, 1); // Append vector to the second dimension

tensor.Print_(); // Print the contents of the tensor

matrix retrieved_matrix = tensor.Get(0); // Get the matrix from the first dimension
Print("Retrieved matrix:", retrieved_matrix);

tensor.Fill(0.0); // Fill the tensor with zeros
tensor.Print_(); // Print the modified tensor

tensor.MemoryClear(); // Release memory used by the tensor
```

This documentation provides a solid foundation for understanding and utilizing the `CTensors` class for handling multidimensional data in your MQL5 projects, particularly within the domain of machine learning.
```