//+------------------------------------------------------------------+
//|                                   Timeseries AI collect data.mq5 |
//|                                     Copyright 2023, Omega Joctan |
//|                        https://www.mql5.com/en/users/omegajoctan |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Omega Joctan"
#property link      "https://www.mql5.com/en/users/omegajoctan"
#property version   "1.00"

#include <MatrixExtend.mqh>
#include <preprocessing.mqh>

StandardizationScaler scaler;

input int training_bars = 10000;

//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//---
   
   vector open, high, low, close;
   open.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_OPEN, 1, training_bars);
   high.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_HIGH,1, training_bars);
   low.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_LOW, 1, training_bars);
   close.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_CLOSE, 1, training_bars);
   
   matrix dataset(training_bars, 4);
   dataset.Col(open, 0);
   dataset.Col(high, 1);
   dataset.Col(low, 2);
   
   vector target_var(training_bars);
   for (ulong i=0; i<(ulong)training_bars; i++)
     {
       if (close[i] > open[i])
         target_var[i] = 1; //Buy 
       else if (close[i] < open[i])
         target_var[i] = -1; //sell
       else
         target_var[i] = 0; //Hold
     }
   
   
   dataset.Col(target_var, 3);
   
//---
   
   matrix x;
   vector y;
   
   if (!MatrixExtend::XandYSplitMatrices(dataset, x, y))
    printf("%s failed to split dataset",__FUNCTION__);
    
    x = scaler.fit_transform(x);
    scaler.save("Standardization-ohlc");
    
    dataset = MatrixExtend::concatenate(x, y);
    
    string header= "open, high, low, close";
    MatrixExtend::WriteCsv("norm-ohlc.csv",dataset, header);
   
  }
//+------------------------------------------------------------------+
