//+------------------------------------------------------------------+
//|                                         nn trainining script.mq5 |
//|                                     Copyright 2023, Omega Joctan |
//|                        https://www.mql5.com/en/users/omegajoctan |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Omega Joctan"
#property link      "https://www.mql5.com/en/users/omegajoctan"
#property version   "1.00"
#property script_show_inputs

#include <regressornets.mqh>
#include <MatrixExtend.mqh>
#include <preprocessing.mqh>

CRegressorNets *nn;
StandardizationScaler scaler;

input int train_bars = 500;


input activation activation_function = AF_LRELU_;
input loss loss_function = LOSS_MSE_;
input double nn_learning_rate = 0.01;
input uint nn_epochs = 1000;
input uint nn_batch_size = 32;
input weight_intializers Weight_initializer = WEIGHT_INTIALIZER_HE;
input regression_metrics metric = METRIC_R_SQUARED;

vector nn_hidden_layers = {10,7}; //single layered nn
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//---
   
   matrix dataset(train_bars, 4);
   vector open, high, low, close;
   
   open.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_OPEN, 1, train_bars);
   high.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_HIGH, 1, train_bars);
   low.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_LOW, 1, train_bars);
   close.CopyRates(Symbol(), PERIOD_CURRENT, COPY_RATES_CLOSE, 1, train_bars);
   
   dataset.Col(open, 0);
   dataset.Col(high, 1);
   dataset.Col(low, 2);
   dataset.Col(close, 3);
   
   matrix x_train, x_test;
   vector y_train, y_test;
   
   MatrixExtend::TrainTestSplitMatrices(dataset, x_train, y_train, x_test, y_test, 0.7, 42);
   
   
   x_train = scaler.fit_transform(x_train);
   scaler.save("ohlc-reg");
   
   nn = new CRegressorNets(nn_hidden_layers, activation_function, loss_function);
   
   nn.fit(x_train, y_train, nn_epochs, new OptimizerSGD(nn_learning_rate), nn_batch_size, false, metric, Weight_initializer);
   
   nn.save_model("regressornets");
   
   delete nn;
  }
//+------------------------------------------------------------------+
